@@ -4,7 +4,7 @@ use File::Path 'mkpath';
 use File::Copy 'copy';
 use Config;
 
-my $VERSION = '1.03';			# Changelog at end
+my $VERSION = '1.04';			# Changelog at end
 die "Debugging cycle detected"		# set to -1 to allow extra iteration
   if ++$ENV{PERL_DEBUG_MCODE_CYCLE} > 1;
 
@@ -32,8 +32,8 @@ arguments (extracted from the generated Makefiles).
 If CHECK-MODULE is empty (or 0), or if FAILING-SCRIPTS are present, 
 rebuilds the current distribution with debugging (in subdirectory $bd),
 and machine-code-debugs Perl crash when running each FAILING-SCRIPT.
-Outputs as much info about the crash as it can massage from gdb or
-dbx.
+Outputs as much info about the crash as it can massage from gdb, or
+dbx, or lldb.
 
 Some minimal intelligence to avoid a flood of useless information is applied:
 if CHECK-MODULE cannot be loaded (but there is no crash during loading), no
@@ -52,7 +52,7 @@ Assumptions:
 	(Actually, v1.00 starts to massage CFLAGS, LDFLAGS and DLLDFLAGS too.)
   If FAILING-SCRIPTs are relative paths, they should be local w.r.t. the
 	root of the distribution.
-  gdb (or dbx) is fresh enough to understand the options we throw in.
+  gdb (or dbx, lldb) is fresh enough to understand the options we throw in.
   Building in a subdirectory does not break a module (e.g., there is
 	no dependence on its position in its parent distribution, if any).
 
@@ -130,7 +130,8 @@ exit 0 unless @ARGV or not $chk_module;
 
 my $gdb = `gdb --version` unless $opt{d};
 my $dbx = `dbx -V -c quit` unless $gdb;
-unless ($gdb or $dbx) {
+my $lldb = `lldb --version` unless $gdb or $dbx;	# untested
+unless ($gdb or $dbx or $lldb) {
   unless ($gdb = `gdb --version`) {
     my($sep, @cand) = quotemeta $Config{path_sep};
     for my $dir (split m($sep), ($ENV{PATH} || '')) {
@@ -143,11 +144,13 @@ unless ($gdb or $dbx) {
 }
 
 sub report_no_debugger () {
-  die "Can't find gdb or dbx" unless defined $gdb or defined $dbx;
+  die "Can't find gdb or dbx" unless defined $gdb or defined $dbx or defined $lldb;
   die "Can't parse output of gdb --version"
-    unless $dbx or $gdb =~ /\b GDB \b | \b Copyright \b .* \b Free Software \b/x;
+    unless $dbx or $lldb or $gdb =~ /\b GDB \b | \b Copyright \b .* \b Free Software \b/x;
   die "Can't parse output of `dbx -V -c quit'"
-    unless $gdb or $dbx =~ /\b dbx \s+ debugger \b/xi;
+    unless $gdb or $lldb or $dbx =~ /\b dbx \s+ debugger \b/xi;
+  warn "Can't parse output of lldb --version: {{{$lldb}}}"
+    unless $dbx or $gdb or $lldb =~ /\b LLDB \b/x;
 }
 
 $@ = '';
@@ -189,9 +192,9 @@ if ($opt{O}) {			# Do not change debugging
 if ($more or $subst) {
   my $FL;
   $subst++ if ($FL = $Config{ccflags}) =~ s/(?<!\S)-(s|O(\d*|[a-z]?)|fomit-frame-pointer)(?!\S)//g;
-  push @extraflags, qq("CCFLAGS=$FL") if $subst;
+  push @extraflags, qq(CCFLAGS=$FL) if $subst;
   for my $f (qw(ldflags lddlflags)) {
-    push @extraflags, qq("\U$f\E=$FL")
+    push @extraflags, qq(\U$f\E=$FL)
       if ($FL = $Config{$f}) =~ s/(?<!\S)-s(?!\S)//g;
   }
 }
@@ -248,6 +251,51 @@ EOP
 
     #open STDIN, $gdb_in or die "cannot open STDIN from $gdb_in: $!";
     @cmd = (qw(gdb -batch), "--command=$gdb_in", $p);
+  } elsif ($lldb) {
+    $ver = $lldb;
+    my $lldb_in = 'lldb-in';
+    open TT, ">$lldb_in" or die "Can't open $lldb_in for write: $!";
+    # bt full: include local vars (not in 5.0; is in 6.5; is in 6.1, but crashes on 6.3:
+	# http://www.cpantesters.org/cpan/report/2fffc390-afd2-11df-834b-ae20f5ac70d3)
+    # disas /m : with source lines (FULL function?!) (not in 6.5; is in 7.0.1)
+    # XXX all-registers may take 6K on amd64; maybe put at end?
+    # sharedlibrary: present on 7.3.1 (2011)
+    my $proc = (-d "/proc/$$" ? <<EOP : '');	# Not working on cygwin 2016, gdb v7.8??? Move to end
+script print "??? info proc mapping"
+script print "\\n=====================================\\n"
+EOP
+    my $extra = '';
+    $extra .= <<EOE if $^O =~ /cygwin|MSWin/;	# Present on 7.3.1 (2011)
+script print "??? info w32 thread-information-block"
+script print "\\n=====================================\\n"
+EOE
+    print TT <<EOP;		# Slightly different order than dbx...
+run -Mblib $script
+script print "\\n=====================================\\n"
+bt
+script print "\\n=====================================\\n"
+frame variable
+script print "\\n=====================================\\n"
+register read
+script print "\\n=====================================\\n"
+disassemble --frame
+script print "\\n=====================================\\n"
+bt 5 full
+script print "\\n=====================================\\n"
+disassemble --frame --mixed
+script print "\\n=====================================\\n"
+image list
+script print "\\n=====================================\\n"
+image dump sections 
+script print "\\n=====================================\\n"
+register read --all
+script print "\\n=====================================\\n"
+${extra}${proc}quit
+EOP
+    close TT or die "Can't close $lldb_in for write: $!";
+
+    #open STDIN, $gdb_in or die "cannot open STDIN from $gdb_in: $!";
+    @cmd = (qw(lldb -batch -s), $lldb_in, $p);
   } else {			# Assume $script has no spaces or metachars
 	# Linux: /proc/$proc/maps has the text map
 	# Solaris: /proc/$proc/map & /proc/$proc/rmap: binary used/reserved
@@ -291,4 +339,6 @@ __END__
 	Report candidates in more situations.
 	Remove unused code.
 	May try to open (failing) scripts in ../ (happens with Math::Pari if the build in a subdirectory "finds" a different version).
-		XXX Propagate flags for Makefile.PL???
\ No newline at end of file
+		XXX Propagate flags for Makefile.PL???
+1.04	Try to support lldb (untested; mapping of /proc and w32 thread-information-block unsupported).
+	Remove extra quoting from the Makefile.PL command-line.
